/*
    WebMaker - KDE HTML Editor
    Copyright (C) 1998, 1999 Alexei Dets <dets@services.ru>
	
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
			
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
					
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <qlayout.h>
#include <qvalidator.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <kapp.h>
#include <kiconloader.h>
#include <kfiledialog.h>
#include "confdialogs.h"

PathsTab::PathsTab(QWidget *parent, const char *name):QWidget(parent, name)
{
		QLabel *label;
		KIconLoader *loader = kapp->getIconLoader();
		QGridLayout *grid=new QGridLayout(this, 7, 2, 5);
	
		browserPath=new QLineEdit(this);
		browserPath->setMinimumSize(browserPath->sizeHint());
		label=new QLabel(browserPath, i18n("Path to external browser (Netscape)"), this);
		label->setMinimumSize(label->sizeHint());
		QPushButton *browserPathButton=new QPushButton(this);
		browserPathButton->setPixmap(loader->loadIcon("fileopen.xpm"));
		browserPathButton->setMinimumSize(browserPathButton->sizeHint());
		connect(browserPathButton, SIGNAL(clicked()), this, SLOT(browserPathClicked()));
		grid->addWidget(label, 0, 0);
		grid->addWidget(browserPath, 1, 0);
		grid->addWidget(browserPathButton, 1, 1);
	
		templatePath=new QLineEdit(this);
		templatePath->setMinimumSize(templatePath->sizeHint());
		label=new QLabel(templatePath, i18n("Path to HTML templates"), this);
		label->setMinimumSize(label->sizeHint());
		QPushButton *templatePathButton=new QPushButton(this);
		templatePathButton->setPixmap(loader->loadMiniIcon("fileopen.xpm"));
		templatePathButton->setMinimumSize(templatePathButton->sizeHint());
		connect(templatePathButton, SIGNAL(clicked()), this, SLOT(templatePathClicked()));
		grid->addWidget(label, 2, 0);
		grid->addWidget(templatePath, 3, 0);
		grid->addWidget(templatePathButton, 3, 1);
	
		treeRoot=new QLineEdit(this);
		treeRoot->setMinimumSize(treeRoot->sizeHint());
		label=new QLabel(treeRoot, i18n("Root of the directory tree"), this);
		label->setMinimumSize(label->sizeHint());
		QPushButton *treeRootButton=new QPushButton(this);
		treeRootButton->setPixmap(loader->loadMiniIcon("fileopen.xpm"));
		treeRootButton->setMinimumSize(treeRootButton->sizeHint());
		connect(treeRootButton, SIGNAL(clicked()), this, SLOT(treeRootClicked()));
		grid->addWidget(label, 4, 0);
		grid->addWidget(treeRoot, 5, 0);
		grid->addWidget(treeRootButton, 5, 1);
	
		grid->setColStretch(0, 1);
		grid->setRowStretch(6, 1);
		grid->activate();
}

void PathsTab::setData(const QString &pathToBrowser,
																							const QString &pathToTemplates,
																							const QString &pathToTreeRoot)
{
		browserPath->setText(pathToBrowser);
		templatePath->setText(pathToTemplates);
		treeRoot->setText(pathToTreeRoot);
}

void PathsTab::browserPathClicked()
{
		QString path=KFileDialog::getOpenFileName(0, 0, this);
		if (path.isEmpty()) return;
		browserPath->setText(path);
}

void PathsTab::templatePathClicked()
{
		QString path=KFileDialog::getOpenFileName(0, 0, this);
		if (path.isEmpty()) return;
		templatePath->setText(path);
}

void PathsTab::treeRootClicked()
{
		QString path=KFileDialog::getOpenFileName(0, 0, this);
		if (path.isEmpty()) return;
		treeRoot->setText(path);
}

FiltersTab::FiltersTab(QWidget *parent, const char *name):QWidget(parent, name)
{
		QLabel *label;
		QVBoxLayout *vbox=new QVBoxLayout(this, 5);
	
		htmlFilterEdit=new QLineEdit(this);
		htmlFilterEdit->setMinimumSize(htmlFilterEdit->sizeHint());
		label=new QLabel(htmlFilterEdit, i18n("HTML files mask"), this);
		label->setMinimumSize(label->sizeHint());
		vbox->addWidget(label);
		vbox->addWidget(htmlFilterEdit);
	
		textFilterEdit=new QLineEdit(this);
		textFilterEdit->setMinimumSize(textFilterEdit->sizeHint());
		label=new QLabel(textFilterEdit, i18n("Text files mask"), this);
		label->setMinimumSize(label->sizeHint());
		vbox->addWidget(label);
		vbox->addWidget(textFilterEdit);
	
		imageFilterEdit=new QLineEdit(this);
		imageFilterEdit->setMinimumSize(imageFilterEdit->sizeHint());
		label=new QLabel(imageFilterEdit, i18n("Image files mask"), this);
		label->setMinimumSize(label->sizeHint());
		vbox->addWidget(label);
		vbox->addWidget(imageFilterEdit);
	
		backupFilterEdit=new QLineEdit(this);
		backupFilterEdit->setMinimumSize(backupFilterEdit->sizeHint());
		label=new QLabel(backupFilterEdit, i18n("Backup files mask"), this);
		label->setMinimumSize(label->sizeHint());
		vbox->addWidget(label);
		vbox->addWidget(backupFilterEdit);
	
		vbox->addStretch(1);
		vbox->activate();
}

void FiltersTab::setData(const QString &htmlFilter,
																									const QString &textFilter,
																									const QString &imageFilter,
																									const QString &backupFilter)
{
		htmlFilterEdit->setText(htmlFilter);
		textFilterEdit->setText(textFilter);
		imageFilterEdit->setText(imageFilter);
		backupFilterEdit->setText(backupFilter);
}

ColorsTab::ColorsTab(QWidget *parent, const char *name):QWidget(parent, name)
{
		QLabel *label;
		QGridLayout *grid=new QGridLayout(this, 9, 2, 5);
		
		anchorButton=new KColorButton(palette[HTMLEdit::ANCHOR], this);
		anchorButton->setMinimumSize(anchorButton->sizeHint());
		label=new QLabel(anchorButton, i18n("Anchor Tags"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addWidget(anchorButton, 0, 1);	
		
		commentButton=new KColorButton(palette[HTMLEdit::COMMENT], this);
		commentButton->setMinimumSize(commentButton->sizeHint());
		label=new QLabel(commentButton, i18n("Comment Tags"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 1, 0);
		grid->addWidget(commentButton, 1, 1);	
		
		styleButton=new KColorButton(palette[HTMLEdit::STYLE], this);
		styleButton->setMinimumSize(styleButton->sizeHint());
		label=new QLabel(styleButton, i18n("Style and Font Tags"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 2, 0);
		grid->addWidget(styleButton, 2, 1);	
		
		imageButton=new KColorButton(palette[HTMLEdit::IMAGE], this);
		imageButton->setMinimumSize(imageButton->sizeHint());
		label=new QLabel(imageButton, i18n("Image Tags"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 3, 0);
		grid->addWidget(imageButton, 3, 1);	
		
		scriptButton=new KColorButton(palette[HTMLEdit::SCRIPT], this);
		scriptButton->setMinimumSize(scriptButton->sizeHint());
		label=new QLabel(scriptButton, i18n("Script and Object Tags"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 4, 0);
		grid->addWidget(scriptButton, 4, 1);	
		
		tableButton=new KColorButton(palette[HTMLEdit::TABLE], this);
		tableButton->setMinimumSize(tableButton->sizeHint());
		label=new QLabel(tableButton, i18n("Table Tags"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 5, 0);
		grid->addWidget(tableButton, 5, 1);	
		
		quotedButton=new KColorButton(palette[HTMLEdit::QUOTED], this);
		quotedButton->setMinimumSize(quotedButton->sizeHint());
		label=new QLabel(quotedButton, i18n("Quoted Values"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 6, 0);
		grid->addWidget(quotedButton, 6, 1);	
		
		defaultButton=new KColorButton(palette[HTMLEdit::DEFAULT], this);
		defaultButton->setMinimumSize(defaultButton->sizeHint());
		label=new QLabel(defaultButton, i18n("Default Color"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 7, 0);
		grid->addWidget(defaultButton, 7, 1);	
		
		grid->setColStretch(0, 1);
		grid->setRowStretch(8, 1);
		grid->activate();
}

void ColorsTab::setData(const TagPalette &colors)
{
		palette=colors;
		anchorButton->setColor(palette[HTMLEdit::ANCHOR]);
		commentButton->setColor(palette[HTMLEdit::COMMENT]);
		styleButton->setColor(palette[HTMLEdit::STYLE]);
		imageButton->setColor(palette[HTMLEdit::IMAGE]);
		scriptButton->setColor(palette[HTMLEdit::SCRIPT]);
		tableButton->setColor(palette[HTMLEdit::TABLE]);
		quotedButton->setColor(palette[HTMLEdit::QUOTED]);
		defaultButton->setColor(palette[HTMLEdit::DEFAULT]);
}

const TagPalette& ColorsTab::tagPalette()
{
		palette[HTMLEdit::ANCHOR]=anchorButton->color();
		palette[HTMLEdit::COMMENT]=commentButton->color();
		palette[HTMLEdit::STYLE]=styleButton->color();
		palette[HTMLEdit::IMAGE]=imageButton->color();
		palette[HTMLEdit::SCRIPT]=scriptButton->color();
		palette[HTMLEdit::TABLE]=tableButton->color();
		palette[HTMLEdit::QUOTED]=quotedButton->color();
		palette[HTMLEdit::DEFAULT]=defaultButton->color();
		return palette;
}

OtherTab::OtherTab(QWidget *parent, const char *name):QWidget(parent, name)
{
		QLabel *label;
		QGridLayout *grid=new QGridLayout(this, 3, 2, 5);
	
		maxRecentCountEdit=new QLineEdit(this);
		maxRecentCountEdit->setMaxLength(2);
		maxRecentCountEdit->setMinimumSize(maxRecentCountEdit->sizeHint());
		QIntValidator *validator=new QIntValidator(0, 99, this);
		maxRecentCountEdit->setValidator(validator);
		label=new QLabel(maxRecentCountEdit, i18n("Recent Files Count (0-99)"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addWidget(maxRecentCountEdit, 0, 1);

		backup=new QCheckBox("Create Backup Files", this);
		backup->setMinimumSize(backup->sizeHint());
		backup->setEnabled(FALSE);      //temporary, until KEdit will be repaired
		grid->addWidget(backup, 1, 0);
		
		grid->setColStretch(0, 1);
		grid->setRowStretch(2, 1);
		grid->activate();
}

void OtherTab::setData(uint maxRecentCount, bool useBackups)
{
		QString s;
		s.setNum(maxRecentCount);
		maxRecentCountEdit->setText(s);
		backup->setChecked(useBackups);
}

uint OtherTab::maxRecentCount()
{
		QString s(maxRecentCountEdit->text());
		return s.toUInt();
}

GeneralConfDialog::GeneralConfDialog(QWidget *parent, const char *name):QTabDialog(parent, name, TRUE)
{
		paths = new PathsTab(this);
		addTab(paths, i18n("&Paths"));
		filters = new FiltersTab(this);
		addTab(filters, i18n("&Directory Filters"));
		colors = new ColorsTab(this);
		addTab(colors, i18n("&Colors"));
		other = new OtherTab(this);
		addTab(other, i18n("&Other"));

		setOkButton(i18n("&OK"));
		setCancelButton(i18n("&Cancel"));
		setCaption(i18n("WebMaker: Options"));
}

void GeneralConfDialog::setData(const ConfigData &data)
{
		paths->setData(data.browserPath, data.templatePath, data.treeRoot);
		filters->setData(data.htmlFilter, data.textFilter,
																			data.imageFilter, data.backupFilter);
		colors->setData(data.palette);
		other->setData(data.maxRecentCount, data.useBackups);
}

void GeneralConfDialog::getData(ConfigData &data)
{
		data.browserPath=paths->pathToBrowser();
		data.templatePath=paths->pathToTemplates();
		data.treeRoot=paths->pathToTreeRoot();
		data.htmlFilter=filters->htmlFilter();
		data.textFilter=filters->textFilter();
		data.imageFilter=filters->imageFilter();
		data.backupFilter=filters->backupFilter();
		data.palette=colors->tagPalette();
		data.maxRecentCount=other->maxRecentCount();
		data.useBackups=other->useBackups();
}
