/*
    WebMaker - KDE HTML Editor
    Copyright (C) 1998, 1999 Alexei Dets <dets@services.ru>
	
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
			
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
					
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/    

#ifndef _CONF_DIALOGS_H_

#define _CONF_DIALOGS_H_

#include <qlineedit.h>
#include <qcheckbox.h>
#include <qtabdialog.h>
#include <kcolorbtn.h>
#include "htmledit.moc.h"

struct ConfigData
{
		QString browserPath;
		QString templatePath;
		QString treeRoot;           //Root of the directory tree
 	QString htmlFilter;         //File masks for directory tree
		QString textFilter;
		QString imageFilter;
		QString backupFilter;
		bool pixmapPrinting;        //Dirty hack to allow "WYSIWYG" printing
		uint maxRecentCount;        //Amount of files in "Recent Files" menu
		bool useBackups;            //Create backup files or not
		TagPalette palette;
};

class PathsTab: public QWidget
{
 Q_OBJECT
public:
	PathsTab(QWidget *parent=0, const char *name=0);
	void setData(const QString &pathToBrowser,
														const QString &pathToTemplates,
														const QString &pathToTreeRoot);
	inline QString pathToBrowser()	{return browserPath->text();};
	inline QString pathToTemplates()	{return templatePath->text();};
	inline QString pathToTreeRoot()	{return treeRoot->text();};
protected:
 QLineEdit *browserPath, *templatePath, *treeRoot;
protected slots:
	void browserPathClicked();
	void templatePathClicked();
	void treeRootClicked();
};

class FiltersTab: public QWidget
{
public:
	FiltersTab(QWidget *parent=0, const char *name=0);
	void setData(const QString &htmlFilter,
														const QString &textFilter,
														const QString &imageFilter,
														const QString &backupFilter);
	inline QString htmlFilter()	{return htmlFilterEdit->text();};
	inline QString textFilter()	{return textFilterEdit->text();};
	inline QString imageFilter()	{return imageFilterEdit->text();};
	inline QString backupFilter()	{return backupFilterEdit->text();};
protected:
 QLineEdit *htmlFilterEdit, *textFilterEdit;
 QLineEdit *imageFilterEdit, *backupFilterEdit;
};

class ColorsTab: public QWidget
{
public:
	ColorsTab(QWidget *parent=0, const char *name=0);
	void setData(const TagPalette &colors);
	const TagPalette& tagPalette();
protected:
	TagPalette palette;
	KColorButton	*anchorButton, *commentButton, *styleButton, *imageButton;
	KColorButton	*scriptButton, *tableButton, *quotedButton, *defaultButton;
};

class OtherTab: public QWidget
{
public:
	OtherTab(QWidget *parent=0, const char *name=0);
	void setData(uint maxRecentCount, bool useBackups);
	uint maxRecentCount();
	inline bool useBackups() {return backup->isChecked();};
protected:
 QLineEdit *maxRecentCountEdit;
	QCheckBox *backup;
};

class GeneralConfDialog: public QTabDialog
{
public:
	GeneralConfDialog(QWidget *parent=0, const char *name=0);
	void setData(const ConfigData &data);
	void getData(ConfigData &data);
protected:
 PathsTab *paths;
	FiltersTab *filters;
	OtherTab *other;
	ColorsTab *colors;
};

#endif
