/*
    WebMaker - KDE HTML Editor
    Copyright (C) 1998, 1999 Alexei Dets <dets@services.ru>
	
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
			
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
					
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/    

//#include <iostream.h>
#include <config.h>
#include <qclipboard.h>

//Next line must be defined _ONLY_ in CVS repository
//#define HAVE_KIMGIO 1

#ifdef HAVE_KIMGIO
#include <kimgio.h>
#else
#ifdef HAVE_QIMGIO
#include <qimageio.h>
#else
#error WebMaker requires kimgio or qimgio for compilation!
#endif
#endif

#include "qextfileinfo.h"
#include <kiconloader.h>
#include <kkeydialog.h>
#include <kmsgbox.h>
#include <kfiledialog.h>
#include <kurl.h>
#include <kdir.h>
#include "confdialogs.h"
#include "webmaker.moc.h"

// Constructor of Application

HTMLEditor::HTMLEditor(const char *name):KTMainWindow(name)
{
#ifdef HAVE_KIMGIO
		kimgioRegister();
#else
		qInitImageIO();
#endif				
		setCaption("WebMaker");
		ReadSettings();
		InitAccelerators();
		InitMenuBar();
		InitStatusBar();
		InitMainWidget();
		InitMainToolBar();
		InitTagToolBars();
		InitTimer();
		ProcessParameters();
}

HTMLEditor::~HTMLEditor()
{
		delete StatusBarTimer;
		delete toolBar(toolBars[tablesToolBar]);
		delete toolBar(toolBars[scriptsToolBar]);
		delete toolBar(toolBars[listsToolBar]);
		delete toolBar(toolBars[framesToolBar]);
		delete toolBar(toolBars[formsToolBar]);
		delete toolBar(toolBars[fontsToolBar]);
		delete toolBar(toolBars[generalToolBar]);
		delete toolBar(toolBars[mainToolBar]);
		delete MainWidget;
		delete statusBar();
		delete menuBar();
		delete keys;
		delete EditorsList;
		delete EditorMenu;
		delete toolBarEntryList;
		delete pluginList;
		delete recentFilesList;
}

bool HTMLEditor::eventFilter(QObject*, QEvent *ev)
{
		if (ev->type() == Event_MouseButtonPress) 
				{
						QMouseEvent *e = (QMouseEvent *)ev;
						if (e->button() == RightButton) 
								{
										//     QMouseEvent trickEvent(Event_MouseButtonPress, e->pos(), LeftButton, LeftButton);
										//     kapp->sendEvent(o, &trickEvent);
								};
				};
		return FALSE; 
}

void HTMLEditor::InitStatusBar()
{
		KStatusBar *StatusBar = new KStatusBar(this);
		StatusBar->setInsertOrder(KStatusBar::RightToLeft);
		StatusBar->insertItem("Line: 000000 Col: 000", ID_STATUS_LINE_COL);
		StatusBar->insertItem("XXX", ID_STATUS_INS);
		StatusBar->setAlignment(ID_STATUS_INS, AlignCenter);
		StatusBar->insertItem("", ID_STATUS_MAIN);
	
		StatusBar->changeItem("Line: 1 Col: 1", ID_STATUS_LINE_COL);
		StatusBar->changeItem("INS", ID_STATUS_INS);
	
		setStatusBar(StatusBar);
}

void HTMLEditor::InitAccelerators()
{
		keys=new KAccel(this);
		
		/*-------------File Menu-------------*/
		keys->insertStdItem(KAccel::New);
  keys->insertItem(i18n("New From Template"), "New From Template", "");
		keys->insertStdItem(KAccel::Open);
		keys->insertStdItem(KAccel::Save);
  keys->insertItem(i18n("Save As"), "Save As", SHIFT + CTRL + Key_S);
  keys->insertItem(i18n("Save All"), "Save All", "");
		keys->insertStdItem(KAccel::Close);
  keys->insertItem(i18n("Close All"), "Close All", "");
		keys->insertStdItem(KAccel::Print);
		keys->insertStdItem(KAccel::Quit, "Exit");

		keys->connectItem(KAccel::New, this, SLOT(NewFile()));
		keys->connectItem("New From Template", this, SLOT(NewFileFromTemplate()));
		keys->connectItem(KAccel::Open, this, SLOT(OpenFile()));
		keys->connectItem(KAccel::Save, this, SLOT(SaveFile()));
		keys->connectItem("Save As", this, SLOT(SaveFileAs()));
		keys->connectItem("Save All", this, SLOT(SaveAllFiles()));
		keys->connectItem(KAccel::Close, this, SLOT(Close()));
		keys->connectItem("Close All", this, SLOT(CloseAll()));
		keys->connectItem(KAccel::Print, this, SLOT(Print()));
		keys->connectItem(KAccel::Quit, this, SLOT(Exit()));

		/*------------Edit Menu--------------*/
		keys->insertStdItem(KAccel::Cut);
		keys->insertStdItem(KAccel::Copy);
		keys->insertStdItem(KAccel::Paste);
  keys->insertItem(i18n("Clear"), "Clear", "");
  keys->insertItem(i18n("Select All"), "Select All", "");
		keys->insertStdItem(KAccel::Find);
		keys->insertStdItem(KAccel::Replace);
  keys->insertItem(i18n("Find Again"), "Find Again", "");
  keys->insertItem(i18n("Go to Line"), "Go to Line", CTRL + Key_G);
		
		keys->connectItem(KAccel::Cut, this, SLOT(Cut()));
		keys->connectItem(KAccel::Copy, this, SLOT(Copy()));
		keys->connectItem(KAccel::Paste, this, SLOT(Paste()));
		keys->connectItem("Clear", this, SLOT(Clear()));
		keys->connectItem("Select All", this, SLOT(SelectAll()));
		keys->connectItem(KAccel::Find, this, SLOT(Search()));
		keys->connectItem(KAccel::Replace, this, SLOT(Replace()));
		keys->connectItem("Find Again", this, SLOT(SearchAgain()));
		keys->connectItem("Go to Line", this, SLOT(GotoLine()));
		
		/*------------Tools Menu-------------*/
  keys->insertItem(i18n("Tag Chooser"), "Tag Chooser", CTRL + Key_E);
  keys->insertItem(i18n("Edit Tag"), "Edit Tag", Key_F4);
  keys->insertItem(i18n("Table Template"), "Table Template", "");
		
		keys->connectItem("Tag Chooser", this, SLOT(TagChooser()));
		keys->connectItem("Edit Tag", this, SLOT(EditTag()));
		keys->connectItem("Table Template", this, SLOT(TableTemplate()));
		
		/*-------------Tags Menu-------------*/
  keys->insertItem(i18n("Bold"), "Bold", SHIFT + CTRL + Key_B);
  keys->insertItem(i18n("Italic"), "Italic", SHIFT + CTRL + Key_I);
  keys->insertItem(i18n("Underline"), "Underline", SHIFT + CTRL + Key_U);
  keys->insertItem(i18n("Font Size +1"), "Font Size +1", CTRL + Key_Equal);
  keys->insertItem(i18n("Font Size -1"), "Font Size -1", CTRL + Key_Minus);
  keys->insertItem(i18n("Quote"), "Quote", SHIFT + CTRL + Key_Q);
  keys->insertItem(i18n("Center"), "Center", SHIFT + CTRL + Key_C);
  keys->insertItem(i18n("Horizontal Rule"), "Horizontal Rule", SHIFT + CTRL + Key_H);
  keys->insertItem(i18n("Non-breaking Space"), "Non-breaking Space", SHIFT + Key_Space);
  keys->insertItem(i18n("Line Break"), "Line Break", SHIFT + Key_Return);
  keys->insertItem(i18n("Line Break "), "Line_Break", SHIFT + Key_Enter);
  keys->insertItem(i18n("Paragraph"), "Paragraph", SHIFT + CTRL + Key_P);
  keys->insertItem(i18n("H1"), "H1", CTRL + Key_1);
  keys->insertItem(i18n("H2"), "H2", CTRL + Key_2);
  keys->insertItem(i18n("H3"), "H3", CTRL + Key_3);
		
		keys->connectItem("Bold", this, SLOT(Bold()));
		keys->connectItem("Italic", this, SLOT(Italic()));
		keys->connectItem("Underline", this, SLOT(Underline()));
		keys->connectItem("Font Size +1", this, SLOT(IncreaseFont()));
		keys->connectItem("Font Size -1", this, SLOT(DecreaseFont()));
		keys->connectItem("Quote", this, SLOT(Quote()));
		keys->connectItem("Center", this, SLOT(Center()));
		keys->connectItem("Horizontal Rule", this, SLOT(HR()));
		keys->connectItem("Non-breaking Space", this, SLOT(nbsp()));
		keys->connectItem("Line Break", this, SLOT(LineBreak()));
		keys->connectItem("Line_Break", this, SLOT(LineBreak()));
		keys->connectItem("Paragraph", this, SLOT(Paragraph()));
		keys->connectItem("H1", this, SLOT(H1()));
		keys->connectItem("H2", this, SLOT(H2()));
		keys->connectItem("H3", this, SLOT(H3()));
		
		/*-------------View Menu-------------*/
  keys->insertItem(i18n("View in External Browser"), "View in External Browser", Key_F11);
		
		keys->connectItem("View in External Browser", this, SLOT(ExternalBrowser()));
		
		/*------------Options Menu-----------*/
  keys->insertItem(i18n("Save Options"), "Save Options", "");
		
		keys->connectItem("Save Options", this, SLOT(SaveSettings()));

		/*------------Other keys (not from menu)-----------*/
  keys->insertItem(i18n("Next Window"), "Next Window", CTRL + Key_F6);
		keys->connectItem("Next Window", this, SLOT(NextWindow()));
  keys->insertItem(i18n("Previous Window"), "Previous Window", SHIFT + CTRL + Key_F6);
		keys->connectItem("Previous Window", this, SLOT(PreviousWindow()));

		keys->readSettings();
}

void HTMLEditor::InitMenuBar()
{
		unsigned int i;
		int id;
		
		KMenuBar *MenuBar = new KMenuBar(this);
	
		file = new QPopupMenu();
		id = file->insertItem(i18n("Ne&w"), this, SLOT(NewFile()));
		keys->changeMenuAccel(file, id, KAccel::New);
		id = file->insertItem(i18n("New From &Template..."), this, SLOT(NewFileFromTemplate()));
		keys->changeMenuAccel(file, id, "New From Template");
		id = file->insertItem(i18n("&Open..."), this, SLOT(OpenFile()));
		keys->changeMenuAccel(file, id, KAccel::Open);

		recent = new QPopupMenu();
	 for (i=0; i<recentFilesList->count(); i++)
    recent->insertItem(recentFilesList->at(i));
  file->insertItem (i18n("Open &Recent"), recent);
  connect(recent, SIGNAL(activated(int)), SLOT(OpenRecentFile(int)));
		
		ID_Save = file->insertItem(i18n("&Save"), this, SLOT(SaveFile()));
		keys->changeMenuAccel(file, ID_Save, KAccel::Save);
		ID_SaveAs = file->insertItem(i18n("Save &as..."), this, SLOT(SaveFileAs()));
		keys->changeMenuAccel(file, ID_SaveAs, "Save As");
		ID_SaveAll = file->insertItem(i18n("Sa&ve All"), this, SLOT(SaveAllFiles()));
		keys->changeMenuAccel(file, ID_SaveAll, "Save All");
		id = file->insertItem(i18n("&Close"), this, SLOT(Close()));
		keys->changeMenuAccel(file, id, KAccel::Close);
		id = file->insertItem(i18n("C&lose All"), this, SLOT(CloseAll()));
		keys->changeMenuAccel(file, id, "Close All");
		file->insertSeparator();
		file->insertItem(i18n("&Insert..."), this, SLOT(InsertFile()));
		file->insertSeparator();
	
		PluginInfo *plugin;
	
		for (i=0;i<pluginList->count(); i++)
				{
						plugin=pluginList->at(i);
						if (plugin->type==PluginInfo::IMPORT)
								plugin->ID=file->insertItem(i18n(plugin->name));
				};
	
		file->insertSeparator();
	
		for (i=0;i<pluginList->count(); i++)
				{
						plugin=pluginList->at(i);
						if (plugin->type==PluginInfo::EXPORT)
								plugin->ID=file->insertItem(i18n(plugin->name));
				};
	
		file->insertSeparator();
		ID_Print = file->insertItem(i18n("&Print..."), this, SLOT(Print()));
		keys->changeMenuAccel(file, ID_Print, KAccel::Print);
		file->insertSeparator();
		id = file->insertItem(i18n("E&xit"), this, SLOT(Exit()));
		keys->changeMenuAccel(file, id, KAccel::Quit);
		MenuBar->insertItem(i18n("&File"), file);
	
		edit = new QPopupMenu();
		ID_Cut = edit->insertItem(i18n("Cu&t"), this, SLOT(Cut()));
		keys->changeMenuAccel(edit, ID_Cut, KAccel::Cut);
		ID_Copy = edit->insertItem(i18n("&Copy"), this, SLOT(Copy()));
		keys->changeMenuAccel(edit, ID_Copy, KAccel::Copy);
		ID_Paste = edit->insertItem(i18n("&Paste"), this, SLOT(Paste()));
		keys->changeMenuAccel(edit, ID_Paste, KAccel::Paste);
		id = edit->insertItem(i18n("C&lear"), this, SLOT(Clear()));
		keys->changeMenuAccel(edit, id, "Clear");
		id = edit->insertItem(i18n("&Select All"), this, SLOT(SelectAll()));
		keys->changeMenuAccel(edit, id, "Select All");
		edit->insertSeparator();
		id = edit->insertItem(i18n("&Find..."), this, SLOT(Search()));
		keys->changeMenuAccel(edit, id, KAccel::Find);
		id = edit->insertItem(i18n("&Replace..."), this, SLOT(Replace()));
		keys->changeMenuAccel(edit, id, KAccel::Replace);
		id = edit->insertItem(i18n("Find &Again"), this, SLOT(SearchAgain()));
		keys->changeMenuAccel(edit, id, "Find Again");
		edit->insertSeparator();
		id = edit->insertItem(i18n("&Go to Line..."), this, SLOT(GotoLine()));
		keys->changeMenuAccel(edit, id, "Go to Line");
		MenuBar->insertItem(i18n("&Edit"), edit);
	
		tools = new QPopupMenu();
		ID_InsertTag = tools->insertItem(i18n("Tag &Chooser"), this, SLOT(TagChooser()));
		keys->changeMenuAccel(tools, ID_InsertTag, "Tag Chooser");
		ID_EditTag = tools->insertItem(i18n("&Edit Tag..."), this, SLOT(EditTag()));
		keys->changeMenuAccel(tools, ID_EditTag, "Edit Tag");
		ID_TableTemplate = tools->insertItem(i18n("&Table Template..."), this, SLOT(TableTemplate()));
		keys->changeMenuAccel(tools, ID_TableTemplate, "Table Template");
		tools->insertSeparator();
	
		for (i=0;i<pluginList->count(); i++)
				{
						plugin=pluginList->at(i);
						if (plugin->type==PluginInfo::FILTER)
								plugin->ID=tools->insertItem(i18n(plugin->name));
				};
	
		MenuBar->insertItem(i18n("&Tools"), tools);
	
		tags = new QPopupMenu();
		id = tags->insertItem(i18n("Bold"), this, SLOT(Bold()));
		keys->changeMenuAccel(tags, id, "Bold");
		id = tags->insertItem(i18n("Italic"), this, SLOT(Italic()));
		keys->changeMenuAccel(tags, id, "Italic");
		id = tags->insertItem(i18n("Underline"), this, SLOT(Underline()));
		keys->changeMenuAccel(tags, id, "Underline");
		tags->insertSeparator();
		id = tags->insertItem(i18n("Font Size +1"), this, SLOT(IncreaseFont()));
		keys->changeMenuAccel(tags, id, "Font Size +1");
		id = tags->insertItem(i18n("Font Size -1"), this, SLOT(DecreaseFont()));
		keys->changeMenuAccel(tags, id, "Font Size -1");
		tags->insertSeparator();
		id = tags->insertItem(i18n("Quote"), this, SLOT(Quote()));
		keys->changeMenuAccel(tags, id, "Quote");
		id = tags->insertItem(i18n("Center"), this, SLOT(Center()));
		keys->changeMenuAccel(tags, id, "Center");
		id = tags->insertItem(i18n("Horizontal Rule"), this, SLOT(HR()));
		keys->changeMenuAccel(tags, id, "Horizontal Rule");
		id = tags->insertItem(i18n("Non-breaking Space"), this, SLOT(nbsp()));
		keys->changeMenuAccel(tags, id, "Non-breaking Space");
		id = tags->insertItem(i18n("Line Break"), this, SLOT(LineBreak()));
		keys->changeMenuAccel(tags, id, "Line Break");
		id = tags->insertItem(i18n("Paragraph"), this, SLOT(Paragraph()));
		keys->changeMenuAccel(tags, id, "Paragraph");
		tags->insertSeparator();
		id = tags->insertItem(i18n("H1"), this, SLOT(H1()));
		keys->changeMenuAccel(tags, id, "H1");
		id = tags->insertItem(i18n("H2"), this, SLOT(H2()));
		keys->changeMenuAccel(tags, id, "H2");
		id = tags->insertItem(i18n("H3"), this, SLOT(H3()));
		keys->changeMenuAccel(tags, id, "H3");
		MenuBar->insertItem(i18n("T&ags"), tags);
	
		QPopupMenu* view = new QPopupMenu();
		id = view->insertItem(i18n("&View in External Browser"), this, SLOT(ExternalBrowser()));
		keys->changeMenuAccel(view, id, "View in External Browser");
		MenuBar->insertItem(i18n("&View"), view);
	
		options = new QPopupMenu();
		options->setCheckable(TRUE);
	
		toolbars=new QPopupMenu();
		toolbars->setCheckable(TRUE);
		ID_MainToolBar = toolbars->insertItem(i18n("Main Toolbar"), this, SLOT(ToggleMainToolBar()));
		ID_GeneralToolBar = toolbars->insertItem(i18n("General Toolbar"), this, SLOT(ToggleGeneralToolBar()));
		toolbars->insertSeparator();
		ID_AlternateSwitching = toolbars->insertItem(i18n("Alternate Toolbar Switching"), this, SLOT(ToggleAlternateSwitching()));
		toolbars->setItemChecked(ID_MainToolBar, showToolBar[mainToolBar]);
		toolbars->setItemChecked(ID_GeneralToolBar, showToolBar[generalToolBar]);
		toolbars->setItemChecked(ID_AlternateSwitching, toolBarAlternateSwitching);
		options->insertItem(i18n("&Toolbars"), toolbars);
	
		options->insertItem(i18n("&General..."), this, SLOT(GeneralOptions()));
		options->insertItem(i18n("&Font..."), this, SLOT(ChangeFont()));
		options->insertItem(i18n("Configure &Keys..."), this, SLOT(ConfigureKeys()));
		options->insertSeparator();
		ID_LowerCaseTags = options->insertItem(i18n("Lowercase Tags"), this, SLOT(ToggleTagsCase()));
		options->setItemChecked(ID_LowerCaseTags, lowerCaseTags);
		ID_CloseAllTags = options->insertItem(i18n("Close All Tags"), this, SLOT(ToggleTagsClosing()));
		options->setItemChecked(ID_CloseAllTags, closeAllTags);
		ID_AutoIndent = options->insertItem(i18n("Auto &Indent"), this, SLOT(ToggleAutoIndent()));
		options->setItemChecked(ID_AutoIndent, indentMode);
		ID_ShowAllFiles = options->insertItem(i18n("Show All Files"), this, SLOT(ToggleFileView()));
		options->setItemChecked(ID_ShowAllFiles, showAllFiles);
		options->insertSeparator();
		ID_BrowseTempFile = options->insertItem(i18n("Browse Temporary File"), this, SLOT(ToggleBrowseTemp()));
		options->setItemChecked(ID_BrowseTempFile, browseTempCopy);
		int item=options->insertItem(i18n("&External Browsers..."), this, SLOT(DefaultHandler()));
		options->setItemEnabled(item, FALSE);
		options->insertSeparator();
		ID_AutoSaveOptions = options->insertItem(i18n("Auto Save Options"), this, SLOT(ToggleAutoSave()));
		options->setItemChecked(ID_AutoSaveOptions, autoSaveOptions);
		id = options->insertItem(i18n("&Save Options"), this, SLOT(SaveSettings()));
		keys->changeMenuAccel(options, id, "Save Options");
		MenuBar->insertItem(i18n("&Options"), options);

		windows=new QPopupMenu();
		MenuBar->insertItem(i18n("&Windows"), windows);
		
		MenuBar->insertSeparator();
	
		QPopupMenu* help;
		QString helpText;
		helpText.sprintf("%s - KDE HTML Editor\nVersion %s\n\nCopyright 1998, 1999\nAlexei Dets <dets@services.ru>",
																			APP_NAME, APP_VERSION);
		help = kapp->getHelpMenu(TRUE, i18n(helpText));
		MenuBar->insertItem( i18n("&Help"), help);

		connect(file, SIGNAL(aboutToShow()), this, SLOT(UpdateMenuBar()));
		connect(edit, SIGNAL(aboutToShow()), this, SLOT(UpdateMenuBar()));
		connect(tools, SIGNAL(aboutToShow()), this, SLOT(UpdateMenuBar()));
		connect(tags, SIGNAL(aboutToShow()), this, SLOT(UpdateMenuBar()));
		connect(view, SIGNAL(aboutToShow()), this, SLOT(UpdateMenuBar()));
		connect(options, SIGNAL(aboutToShow()), this, SLOT(UpdateMenuBar()));
		connect(windows, SIGNAL(aboutToShow()), this, SLOT(UpdateMenuBar()));
		connect(file, SIGNAL(activated(int)), this, SLOT(StartPlugin(int)));
		connect(tools, SIGNAL(activated(int)), this, SLOT(StartPlugin(int)));
  connect(windows, SIGNAL(activated(int)), SLOT(ChangeWindow(int)));
		setMenu(MenuBar);

		/*================Init editor RMB menu=====================*/
		
		EditorMenu = new QPopupMenu();
		ID_EditorInsertTag = EditorMenu->insertItem(i18n("&Tag Chooser"), this, SLOT(TagChooser()));
		ID_EditorEditTag = EditorMenu->insertItem(i18n("&Edit tag..."), this, SLOT(EditTag()));
		EditorMenu->insertSeparator();
		efile = new QPopupMenu();
		efile->insertItem(i18n("Ne&w..."), this, SLOT(NewFile()));
		efile->insertItem(i18n("&Open..."), this, SLOT(OpenFile()));
		ID_EditorSave=efile->insertItem(i18n("&Save"), this, SLOT(SaveFile()));
		ID_EditorSaveAs=efile->insertItem(i18n("S&ave as..."), this, SLOT(SaveFileAs()));
		efile->insertItem(i18n("&Close"), this, SLOT(Close()));
		EditorMenu->insertItem(i18n("&File"), efile);
		EditorMenu->insertSeparator();
		ID_EditorCut = EditorMenu->insertItem(i18n("Cu&t"), this, SLOT(Cut()));
		ID_EditorCopy = EditorMenu->insertItem(i18n("&Copy"), this, SLOT(Copy()));
		ID_EditorPaste = EditorMenu->insertItem(i18n("&Paste"), this, SLOT(Paste()));
		connect(EditorMenu, SIGNAL(aboutToShow()), this, SLOT(UpdateMenuBar()));
}

void HTMLEditor::InitMainToolBar()
{
		KIconLoader *loader = kapp->getIconLoader();
		QPixmap pixmap;
	
		KToolBar *ToolBar = new KToolBar(this);
		pixmap = loader->loadIcon("filenew.xpm");
		ToolBar->insertButton(pixmap, 0,
																								SIGNAL(clicked()), this,
																								SLOT(NewFile()), TRUE, i18n("New Document"));
		pixmap = loader->loadIcon("fileopen.xpm");
		ToolBar->insertButton(pixmap, 0,
																								SIGNAL(clicked()), this,
																								SLOT(OpenFile()), TRUE, i18n("Open Document"));
		pixmap = loader->loadIcon("filefloppy.xpm");
		ToolBar->insertButton(pixmap, 0,
																								SIGNAL(clicked()), this,
																								SLOT(SaveFile()), TRUE, i18n("Save Document"));
		pixmap = loader->loadIcon("fileprint.xpm");
		ToolBar->insertButton(pixmap, 0,
																								SIGNAL(clicked()), this,
																								SLOT(Print()), TRUE, i18n("Print Document"));
	
		ToolBar->insertSeparator();
		ToolBar->insertSeparator();
	
		pixmap = loader->loadIcon("browse.xpm");
		ToolBar->insertButton(pixmap, 0,
																								SIGNAL(clicked()), this,
																								SLOT(ExternalBrowser()), TRUE, i18n("View in External Browser"));
	
		ToolBar->insertSeparator();
		ToolBar->insertSeparator();
	
		pixmap = loader->loadIcon("editcut.xpm");
		ToolBar->insertButton(pixmap, 0,
																								SIGNAL(clicked()), this,
																								SLOT(Cut()), TRUE, i18n("Cut"));
		pixmap = loader->loadIcon("editcopy.xpm");
		ToolBar->insertButton(pixmap, 0,
																								SIGNAL(clicked()), this,
																								SLOT(Copy()), TRUE, i18n("Copy"));
		pixmap = loader->loadIcon("editpaste.xpm");
		ToolBar->insertButton(pixmap, 0,
																								SIGNAL(clicked()), this,
																								SLOT(Paste()), TRUE, i18n("Paste"));
	
		ToolBar->insertSeparator();
		ToolBar->insertSeparator();

		pixmap = loader->loadIcon("search.xpm");
		ToolBar->insertButton(pixmap, 0,
																								SIGNAL(clicked()), this,
																								SLOT(Search()), TRUE, i18n("Search"));

		ToolBar->insertSeparator();
		ToolBar->insertSeparator();
	
		pixmap = loader->loadIcon("toolbar_fonts.xpm");
		ToolBar->insertButton(pixmap, fontsToolBar,
																								TRUE, i18n("Toggle Fonts Toolbar On/Off "));
		pixmap = loader->loadIcon("toolbar_forms.xpm");
		ToolBar->insertButton(pixmap, formsToolBar,
																								TRUE, i18n("Toggle Forms Toolbar On/Off "));
		pixmap = loader->loadIcon("toolbar_frames.xpm");
		ToolBar->insertButton(pixmap, framesToolBar,
																								TRUE, i18n("Toggle Frames Toolbar On/Off "));
		pixmap = loader->loadIcon("toolbar_lists.xpm");
		ToolBar->insertButton(pixmap, listsToolBar,
																								TRUE, i18n("Toggle Lists Toolbar On/Off "));
		pixmap = loader->loadIcon("toolbar_scripts.xpm");
		ToolBar->insertButton(pixmap, scriptsToolBar,
																								TRUE, i18n("Toggle Scripts Toolbar On/Off "));
		pixmap = loader->loadIcon("toolbar_tables.xpm");
		ToolBar->insertButton(pixmap, tablesToolBar,
																								TRUE, i18n("Toggle Tables Toolbar On/Off "));
		for (int i=fontsToolBar; i<=tablesToolBar; i++)
				{
						ToolBar->setToggle(i);
						ToolBar->setButton(i, showToolBar[i]);
				};

		pixmap = loader->loadIcon("help.xpm");
		ToolBar->insertButton(pixmap, ID_HELP,
																								SIGNAL(clicked()), this,
																								SLOT(Help()), TRUE, i18n("Help"));
		ToolBar->alignItemRight(ID_HELP);
		
		ToolBar->setBarPos( KToolBar::Top );

		connect(ToolBar, SIGNAL(toggled(int)), this, SLOT(ToggleTagToolBars(int)));
		
		toolBars[mainToolBar]=addToolBar(ToolBar);
		if (showToolBar[mainToolBar])
				enableToolBar(KToolBar::Show, toolBars[mainToolBar]);
		else
				enableToolBar(KToolBar::Hide, toolBars[mainToolBar]);
}

void HTMLEditor::InitTagToolBars()
{
		KIconLoader *loader = kapp->getIconLoader();
		QPixmap pixmap;
		KToolBar *ToolBar;
		for (int i=generalToolBar; i<=tablesToolBar; i++)
				{
						ToolBar = new KToolBar(this);
						ToolBar->setBarPos(KToolBar::Top);
						connect(ToolBar, SIGNAL(clicked(int)), this, SLOT(ProcessToolBar(int)));
						toolBars[i]=addToolBar(ToolBar);
						if (showToolBar[i])
								enableToolBar(KToolBar::Show, toolBars[i]);
						else
								enableToolBar(KToolBar::Hide, toolBars[i]); 
				};
		
		uint current;
		for (ToolBarEntry *entry=toolBarEntryList->first(); entry; entry=toolBarEntryList->next())
				{
						current=toolBarEntryList->at();
						if (entry->tag=="---")
								{
										toolBar(toolBars[entry->toolBarID])->insertSeparator();
										toolBar(toolBars[entry->toolBarID])->insertSeparator();
								}
						else
								{
										pixmap=loader->loadIcon(entry->pixmap);
										toolBar(toolBars[entry->toolBarID])->
												insertButton(pixmap, current, TRUE, entry->toolTip);
										(toolBar(toolBars[entry->toolBarID])->getButton(current))->
												setName(entry->tag);
										entry->buttonID=current;
								};
				};

		QStrList *fontsList=new QStrList();
		kapp->getKDEFonts(fontsList);
		QComboBox *fonts=new QComboBox(TRUE, toolBar(toolBars[fontsToolBar]));
		fonts->insertStrList(fontsList);
		fonts->setInsertionPolicy(QComboBox::AtTop);
		fonts->setFocusPolicy(ClickFocus);
		connect(fonts, SIGNAL(activated(const char*)),
										this, SLOT(FontFace(const char*)));
		toolBar(toolBars[fontsToolBar])->insertWidget(0, fonts->sizeHint().width(),
																																																fonts, 0);
		fontsList->clear();
		fontsList->append("-7");
		fontsList->append("-6");
		fontsList->append("-5");
		fontsList->append("-4");
		fontsList->append("-3");
		fontsList->append("-2");
		fontsList->append("-1");
		fontsList->append("+0");
		fontsList->append("+1");
		fontsList->append("+2");
		fontsList->append("+3");
		fontsList->append("+4");
		fontsList->append("+5");
		fontsList->append("+6");
		fontsList->append("+7");
		QComboBox *fontSizes=new QComboBox(TRUE, toolBar(toolBars[fontsToolBar]));
		fontSizes->insertStrList(fontsList);
		fontSizes->setCurrentItem(7);
		fontSizes->setFocusPolicy(ClickFocus);
		connect(fontSizes, SIGNAL(activated(const char*)),
										this, SLOT(FontSize(const char*)));
		toolBar(toolBars[fontsToolBar])->insertWidget(0, fontSizes->sizeHint().width(),
																																																fontSizes, 1);
		delete fontsList;
};

void HTMLEditor::InitMainWidget()
{
		MainWidget=new KNewPanner(this);
		MainWidget->setUnits(KNewPanner::Absolute);
		
		Dirs=new DirView(treeRoot, showAllFiles, MainWidget);
		Dirs->setFilter(HTML, htmlFilter);
		Dirs->setFilter(TEXT, textFilter);
		Dirs->setFilter(IMAGE, imageFilter);
		Dirs->setFilter(BACKUP, backupFilter);
		if (!htmlFilter.isEmpty() || !textFilter.isEmpty() ||
						!imageFilter.isEmpty() || !backupFilter.isEmpty())
				Dirs->setFilter(HTML, 0, TRUE);
		Dirs->setFocusPolicy(ClickFocus);
		
		connect(Dirs, SIGNAL(doubleClicked(QListViewItem*)),
										this, SLOT(OpenFileFromList(QListViewItem*)));
		connect(Dirs, SIGNAL(returnPressed(QListViewItem*)),
										this, SLOT(OpenFileFromList(QListViewItem*)));
		
  Editors=new EditorView(MainWidget);
		EditorsList=new HTMLEditList();
		EditorsList->setAutoDelete(TRUE);
		connect(Editors, SIGNAL(aboutToShow(QWidget*)), this, SLOT(SetCurrentEditor(QWidget*)));
		NewFile();

		//Negative width value == fallback to defaults
		MainWidget->setSeparatorPos((treeWidth>=0)?treeWidth:
																														Dirs->sizeHint().width()*2);
		MainWidget->activate(Dirs, Editors);
		
		setView(MainWidget);
}

void HTMLEditor::InitTimer()
{
		StatusBarTimer = new QTimer(this);
		connect(StatusBarTimer, SIGNAL(timeout()),this,SLOT(RefreshMainStatus()));
}

void HTMLEditor::ProcessParameters()
{
		if (kapp->argc()==1) return;
		
		QTextStream cout(stdout, IO_WriteOnly);
		
		for (int i=1; i<kapp->argc(); i++)
				{
						QString fileName(kapp->argv()[i]);
						
						// This is a hack because in fact,
						// KApp should have removed this parameter.
						// I will check this later again.
						if (fileName=="-restore")
								{
										i++;
										continue;
								};

						if (fileName=="--help" || fileName=="-h")
								{
										cout << APP_NAME << " - KDE HTML Editor\n";
										cout << "Version " << APP_VERSION << "\n";
										cout << "Copyright 1998, 1999, Alexei Dets <dets@services.ru>\n\n";
										cout << "Usage: webmaker [FILE1] [FILE2] ...\n";
										cout << "                --version, -v      - version information\n";
										cout << "                --help, -h         - this help message\n";
										exit(0);
										return;
								};
						if (fileName=="--version" || fileName=="-v")
								{
										cout << APP_NAME << " - KDE HTML Editor\n";
										cout << "Version " << APP_VERSION << "\n";
										cout << "Copyright 1998, 1999, Alexei Dets <dets@services.ru>\n";
										exit(0);
										return;
								};
						QFileInfo file(fileName);
						if (!file.exists() || file.isDir() || !file.isReadable()) continue;
		
						if (Editor->isModified() || Editor->getName()!=i18n("Untitled")) NewFile();
		
						QString currentDir= QDir::currentDirPath();
						int result=Editor->loadFile(file.absFilePath(), KEdit::OPEN_READWRITE);
						QDir::setCurrent(currentDir); // this makes multiple relative pathnames on the commandline work
						if (result!=KEdit::KEDIT_OK) continue; //    !!!
		
						QString s;
						s.sprintf(i18n("File opened: %s"), fileName.data());
						SetMainStatus(s);
						UpdateLineColStatus();
				};
}

void HTMLEditor::NewFile()
{
		Editor = new HTMLEdit( kapp, Editors, "Editor", i18n("Untitled"));
			
		connect(Editor, SIGNAL(CursorPositionChanged()), this, SLOT(UpdateLineColStatus()));
		connect(Editor, SIGNAL(toggle_overwrite_signal()), this, SLOT(ToggleInsStatus()));
		connect(Editor, SIGNAL(fileChanged()), this, SLOT(UpdateCaption()));
		connect(Editor, SIGNAL(loading()), this, SLOT(Loading()));
		connect(Editor, SIGNAL(saving()), this, SLOT(Saving()));

		Editor->installRBPopup(EditorMenu);
		Editor->installEventFilter(this);
	
		Editor->setFont(editorFont);
		Editor->setLowerCaseTags(lowerCaseTags);
		Editor->setCloseAllTags(closeAllTags);
		Editor->setAutoIndentMode(indentMode);
		Editor->saveBackupCopy(useBackups);
		Editor->setExternalBrowser(browser.data());
		Editor->setTagPalette(palette);
	
		Editor->setFillColumnMode(0,FALSE);
		Editor->setWordWrap(FALSE);

		EditorsList->append(Editor);
		Editors->addTab(Editor, i18n("Untitled"));
		Editor->setFocusPolicy(StrongFocus);

		KDNDDropZone *dropZone=new KDNDDropZone(Editor, DndURL);
		connect(dropZone, SIGNAL(dropAction(KDNDDropZone*)),
										this, SLOT(OpenFileFromDrop(KDNDDropZone*)));

		UpdateCaption();
		UpdateLineColStatus();

		Editor->setEnabled(TRUE);
}

void HTMLEditor::NewFileFromTemplate()
{
		if (Editor->isModified() || Editor->getName()!=i18n("Untitled")) NewFile();
		
		int result;
	
		if	(Editor->newFile()==KEdit::KEDIT_USER_CANCEL) return;
		UpdateLineColStatus();
	
 retry:
	
		QString fileName=KFileDialog::getOpenFileName(templatePath, "*.*htm*|HTML Files (*.*htm*)\n*|All Files", this);
		if (fileName.isEmpty()) return;
		result=Editor->loadFile(fileName, KEdit::OPEN_INSERT);
		switch (result)
				{
				case KEdit::KEDIT_OK:
						{
								QString s;
								s.sprintf(i18n("Template opened: %s"), fileName.data());
								SetMainStatus(s); 
								UpdateLineColStatus();
								break;
						};
				case KEdit::KEDIT_RETRY: goto retry; break;
				default: break;
				}
}

void HTMLEditor::OpenFile()
{
		if (Editor->isModified() || Editor->getName()!=i18n("Untitled")) NewFile();
		
		int result;
	
 retry:
	
		result=Editor->openFile(KEdit::OPEN_READWRITE);
		switch (result)
				{
				case KEdit::KEDIT_OK:
						{
								QString s;
								s.sprintf(i18n("File opened: %s"), Editor->getName().data());
								SetMainStatus(s);
								UpdateLineColStatus();
								AddRecentFile(Editor->getName());
								break;
						};
				case KEdit::KEDIT_RETRY: goto retry; break;
				default: break;
				}
}

void HTMLEditor::OpenRecentFile(int id)
{
		QString fileName(recent->text(id));
		QFileInfo file(fileName);
		if (!file.exists() || file.isDir() || !file.isReadable()) return;
		
		if (Editor->isModified() || Editor->getName()!=i18n("Untitled")) NewFile();
		
		int result=Editor->loadFile(fileName, KEdit::OPEN_READWRITE);
		if (result!=KEdit::KEDIT_OK) return; //    !!!
		
		QString s;
		s.sprintf(i18n("File opened: %s"), fileName.data());
		SetMainStatus(s);
		UpdateLineColStatus();
		AddRecentFile(Editor->getName());
}

void HTMLEditor::OpenFileFromList(QListViewItem *item)
{
		QString fileName(Dirs->getFileName(item));
		QExtFileInfo file(fileName);
		if (!file.exists() || file.isDir() || !file.isReadable()) return;
		
		if (KDir::match(imageFilter, file.fileName()))
				{
						file.convertToRelative(Editor->projectRoot());
						Editor->insertTag("IMG",
																								QString(" src=\"") + file.filePath() + '"');
						return;
				};
		if (!KDir::match(htmlFilter, file.fileName()) &&
						!KDir::match(textFilter, file.fileName()) &&
						!KDir::match(backupFilter, file.fileName()))
				{
						file.convertToRelative(Editor->projectRoot());
						Editor->insertTag("A",
																								QString(" href=\"") + file.filePath() + '"');
						return;
				};
		
		if (Editor->isModified() || Editor->getName()!=i18n("Untitled")) NewFile();
		
		int result=Editor->loadFile(fileName, KEdit::OPEN_READWRITE);
		if (result!=KEdit::KEDIT_OK) return; //    !!!
		
		QString s;
		s.sprintf(i18n("File opened: %s"), fileName.data());
		SetMainStatus(s);
		UpdateLineColStatus();
		AddRecentFile(Editor->getName());
}

void HTMLEditor::OpenFileFromDrop(KDNDDropZone *dropZone)
{
		QStrList URLList(dropZone->getURLList());
		KURL url;
		for (uint i=0; i<URLList.count(); i++)
				{
						url=URLList.at(i);
						if (url.isLocalFile())
								{
										QString fileName(url.path());
										QExtFileInfo file(fileName);
										if (!file.exists() || file.isDir() || !file.isReadable()) continue;

										if (KDir::match(imageFilter, file.fileName()))
												{
														file.convertToRelative(Editor->projectRoot());
														Editor->insertTag("IMG",
																																QString(" src=\"") + file.filePath() + '"');
														continue;
												};
										if (!KDir::match(htmlFilter, file.fileName()) &&
														!KDir::match(textFilter, file.fileName()) &&
														!KDir::match(backupFilter, file.fileName()))
												{
														file.convertToRelative(Editor->projectRoot());
														Editor->insertTag("A",
																																QString(" href=\"") + file.filePath() + '"');
														continue;
												};
		
										if (Editor->isModified() || Editor->getName()!=i18n("Untitled")) NewFile();
		
										int result=Editor->loadFile(fileName, KEdit::OPEN_READWRITE);
										if (result!=KEdit::KEDIT_OK) return; //    !!!
		
										QString s;
										s.sprintf(i18n("File opened: %s"), fileName.data());
										SetMainStatus(s);
										UpdateLineColStatus();
										AddRecentFile(Editor->getName());
								}
						else
								{
										KMsgBox::message(this, i18n("Warning"), i18n("Only local files allowed!"));
								}
				};
}

void HTMLEditor::SaveFile()
{
		if (Editor->isModified())
				{
						if (Editor->doSave()==KEdit::KEDIT_OK)
								{
										UpdateCaption();
										QString s;
										s.sprintf(i18n("Saved: %s"), Editor->getName().data());
										SetMainStatus(s);
										AddRecentFile(Editor->getName());
								}
				}
		else
				{
						SetMainStatus(i18n("No changes need to be saved"));
				}
}

void HTMLEditor::SaveFileAs()
{
		if (Editor->saveAs()== KEdit::KEDIT_OK)
				{
						UpdateCaption();
						QString s;
						s.sprintf(i18n("Saved as: %s"), Editor->getName().data());
						SetMainStatus(s);
						AddRecentFile(Editor->getName());
				}
}

void HTMLEditor::SaveAllFiles()
{
		HTMLEdit *currentEditor=Editor;
		Editor=EditorsList->first();
		while (Editor)
				{
						if (Editor->isModified())
								{
										if (Editor->getName()==i18n("Untitled")) Editors->showPage(Editor);
										if (Editor->doSave()==KEdit::KEDIT_OK)
												{
														UpdateCaption();
														QString s;
														s.sprintf(i18n("Saved: %s"), Editor->getName().data());
														SetMainStatus(s);
														AddRecentFile(Editor->getName());
												}
								}
						Editor=EditorsList->next();
				}
		Editors->showPage(currentEditor);
}

void HTMLEditor::Close()
{
		if (Editor->close())
				{
						HTMLEdit *temp=Editor;
						Editors->removeTab(Editor);
						EditorsList->removeRef(temp);
				};
		if (EditorsList->count()==0) NewFile();
}

void HTMLEditor::CloseAll()
{
		bool accept=TRUE;
		HTMLEdit *editor=EditorsList->first();
		while (editor)
				{
						accept=editor->close();
						if (accept)
								{
										HTMLEdit *temp=editor;
						    editor=EditorsList->next();
									 Editors->removeTab(temp);
										EditorsList->removeRef(temp);
								}
						else
								break;
				}
		if (accept) NewFile();
}

void HTMLEditor::Print()
{
  Editor->print(pixmapPrinting);
}

void HTMLEditor::InsertFile()
{
		int result;
	
 retry:
	
		result=Editor->insertFile();
		switch (result)
				{
				case KEdit::KEDIT_OK:	
						{
								SetMainStatus(i18n("File inserted"));
								UpdateLineColStatus();
								break;
						};
				case KEdit::KEDIT_RETRY: goto retry; break;
				default: break;
				};
}

void HTMLEditor::StartPlugin(int ID)
{
		PluginInfo *plugin;
		unsigned int num=pluginList->count();
		for (unsigned int i=0; i<num; i++)
				{
						plugin=pluginList->at(i);
						if (plugin->ID==ID)
								{
										switch(plugin->type)
												{	
												case PluginInfo::IMPORT:
														{
																if (Editor->isModified() || Editor->getName()!=i18n("Untitled")) NewFile();
																Editor->startImport(plugin->commandLine);
																break;
														};
												case PluginInfo::EXPORT: Editor->startExport(plugin->commandLine);break;
												case PluginInfo::FILTER: Editor->startFilter(plugin->commandLine);break;
												};
										break;
								}
				}
}

void HTMLEditor::Exit()
{
		close();
}

void HTMLEditor::closeEvent(QCloseEvent* e)
{
		bool accept=TRUE;
		HTMLEdit *editor=EditorsList->first();
		while (editor)
				{
						accept=editor->close();
						if (accept)
								{
										HTMLEdit *temp=editor;
						    editor=EditorsList->next();
									 Editors->removeTab(temp);
										EditorsList->removeRef(temp);
								}
						else
								break;
				}
		if (accept)
				{
						SaveSettings(TRUE);
						e->accept();
				};
}

void HTMLEditor::Cut()
{
		Editor->cut();
		Editor->toggleModified(TRUE);
		UpdateLineColStatus();
}

void HTMLEditor::Copy()
{
		Editor->copyText();
}

void HTMLEditor::Paste()
{
		Editor->paste();
		Editor->toggleModified(TRUE);
		UpdateLineColStatus();
}

void HTMLEditor::Clear()
{
		Editor->clear();
		Editor->toggleModified(TRUE);
		UpdateLineColStatus();
}

void HTMLEditor::SelectAll()
{
		Editor->selectAll();
}

void HTMLEditor::Search()
{
		Editor->Search();
		UpdateLineColStatus();
}

void HTMLEditor::Replace()
{
		Editor->Replace();
		UpdateLineColStatus();
}

void HTMLEditor::SearchAgain()
{
		Editor->repeatSearch();
		UpdateLineColStatus();
}

void HTMLEditor::GotoLine()
{
		Editor->doGotoLine();
}

void HTMLEditor::TagChooser()
{
		Editor->openTagChooser();
}

void HTMLEditor::EditTag()
{
		Editor->editTag();
}

void HTMLEditor::TableTemplate()
{
		Editor->createTableTemplate();
}

void HTMLEditor::Bold()
{
		Editor->insertTag("B");
}

void HTMLEditor::Italic()
{
		Editor->insertTag("I");
}

void HTMLEditor::Underline()
{
		Editor->insertTag("U");
}

void HTMLEditor::IncreaseFont()
{
		Editor->insertTag("FONT", " size=\"+1\"");
}

void HTMLEditor::DecreaseFont()
{
		Editor->insertTag("FONT", " size=\"-1\"");
}

void HTMLEditor::Quote()
{
		Editor->insertTag("Q");
}

void HTMLEditor::Anchor()
{
		Editor->insertTagWithEdit("A");
}

void HTMLEditor::Image()
{
		Editor->insertTagWithEdit("IMG");
}

void HTMLEditor::Center()
{
		Editor->insertTag("DIV", " align=\"center\"");
}

void HTMLEditor::Left()
{
		Editor->insertTag("DIV", " align=\"left\"");
}

void HTMLEditor::Right()
{
		Editor->insertTag("DIV", " align=\"right\"");
}

void HTMLEditor::Justify()
{
		Editor->insertTag("DIV", " align=\"justify\"");
}

void HTMLEditor::HR()
{
		Editor->insertTag("HR");
}

void HTMLEditor::nbsp()
{
		Editor->insertEntity("&nbsp;");
}

void HTMLEditor::LineBreak()
{
		Editor->insertTag("BR");
}

void HTMLEditor::Paragraph()
{
		Editor->insertTag("P");
}

void HTMLEditor::H1()
{
		Editor->insertTag("H1");
}

void HTMLEditor::H2()
{
		Editor->insertTag("H2");
}

void HTMLEditor::H3()
{
		Editor->insertTag("H3");
}

void HTMLEditor::ExternalBrowser()
{
		Editor->browseExternal(browseTempCopy);
}

void HTMLEditor::ToggleMainToolBar()
{
		showToolBar[mainToolBar]=!showToolBar[mainToolBar];
		enableToolBar(KToolBar::Toggle, toolBars[mainToolBar]);
		toolbars->setItemChecked(ID_MainToolBar, showToolBar[mainToolBar]);
}

void HTMLEditor::ToggleGeneralToolBar()
{
		showToolBar[generalToolBar]=!showToolBar[generalToolBar];
		enableToolBar(KToolBar::Toggle, toolBars[generalToolBar]);
		toolbars->setItemChecked(ID_GeneralToolBar, showToolBar[generalToolBar]);
}

void HTMLEditor::ToggleTagToolBars(int ID)
{
		if (toolBarAlternateSwitching)
				{
						if (showToolBar[ID])
								{
										showToolBar[ID]=FALSE;
										enableToolBar(KToolBar::Hide, toolBars[ID]);
								}
						else
								{
										setUpdatesEnabled(FALSE);
										for (int i=fontsToolBar; i<=tablesToolBar; i++)
												{													
														if (showToolBar[i])
																{
																		toolBar(toolBars[mainToolBar])->setButton(i, FALSE);
																};
												};
										showToolBar[ID]=TRUE;
										enableToolBar(KToolBar::Show, toolBars[ID]);										
										setUpdatesEnabled(TRUE);
										repaint();
								};
				}
		else
				{
						showToolBar[ID]=!showToolBar[ID];
						enableToolBar(KToolBar::Toggle, toolBars[ID]);
				};
};

void HTMLEditor::ToggleAlternateSwitching()
{
		toolBarAlternateSwitching=!toolBarAlternateSwitching;
		toolbars->setItemChecked(ID_AlternateSwitching, toolBarAlternateSwitching);
		if (toolBarAlternateSwitching)
				SetMainStatus(i18n("Alternate Toolbar Switching: On"));
		else
				SetMainStatus(i18n("Alternate Toolbar Switching: Off"));
}

void HTMLEditor::GeneralOptions()
{
		GeneralConfDialog *dialog=new GeneralConfDialog(this);
		if (!dialog) return;
		
		ConfigData data;
		data.browserPath=browser;
		data.templatePath=templatePath;
		data.treeRoot=treeRoot;
		data.htmlFilter=htmlFilter;
		data.textFilter=textFilter;
		data.imageFilter=imageFilter;
		data.backupFilter=backupFilter;
		data.palette=palette;
		data.maxRecentCount=maxRecentCount;
		data.useBackups=useBackups;
		dialog->setData(data);
		
		if (dialog->exec())
				{
						dialog->getData(data);
						browser=data.browserPath;
						templatePath=data.templatePath;
						treeRoot=data.treeRoot;
						htmlFilter=data.htmlFilter;
						textFilter=data.textFilter;
						imageFilter=data.imageFilter;
						backupFilter=data.backupFilter;
						palette=data.palette;
						maxRecentCount=data.maxRecentCount;
						useBackups=data.useBackups;

						Dirs->setFilter(HTML, htmlFilter);
						Dirs->setFilter(TEXT, textFilter);
						Dirs->setFilter(IMAGE, imageFilter);
						Dirs->setFilter(BACKUP, backupFilter);
						Dirs->setRoot(treeRoot);
		
						HTMLEditList tempList(*EditorsList);
						tempList.setAutoDelete(FALSE);
						HTMLEdit *editor=tempList.first();
						while (editor)
								{
										editor->setExternalBrowser(browser.data());
										editor->saveBackupCopy(useBackups);
										editor->setTagPalette(palette);
										editor=tempList.next();
								};
				}
		else
				{
						delete dialog;
						return;
				};
		delete dialog;
}

void HTMLEditor::ChangeFont()
{
		Editor->selectFont();
		editorFont=Editor->font();
}

void HTMLEditor::ConfigureKeys()
{
		KKeyDialog::configureKeys(keys, false);
}

void HTMLEditor::ToggleTagsCase()
{
		lowerCaseTags=!lowerCaseTags;
		Editor->setLowerCaseTags(lowerCaseTags);
		options->setItemChecked(ID_LowerCaseTags, lowerCaseTags);
		if (lowerCaseTags)
				SetMainStatus(i18n("Insert Tags in Lower Case: On"));
		else
				SetMainStatus(i18n("Insert Tags in Lower Case: Off"));
}

void HTMLEditor::ToggleTagsClosing()
{
		closeAllTags=!closeAllTags;
		Editor->setCloseAllTags(closeAllTags);
		options->setItemChecked(ID_CloseAllTags, closeAllTags);
		if (closeAllTags)
				SetMainStatus(i18n("Close All Tags: On"));
		else
				SetMainStatus(i18n("Close All Tags: Off"));
}

void HTMLEditor::ToggleAutoIndent()
{
		indentMode = !indentMode;
		Editor->setAutoIndentMode(indentMode);
		options->setItemChecked(ID_AutoIndent, indentMode);
		if (indentMode)
				SetMainStatus(i18n("Auto Indent Mode: On"));
		else
				SetMainStatus(i18n("Auto Indent Mode: Off"));
}

void HTMLEditor::ToggleFileView()
{
		showAllFiles = !showAllFiles;
		options->setItemChecked(ID_ShowAllFiles, showAllFiles);
		if (showAllFiles)
				SetMainStatus(i18n("Show All Files: On. Updating disk info..."));
		else
				SetMainStatus(i18n("Show All Files: Off. Updating disk info..."));
		Dirs->showAllFiles(showAllFiles);
}

void HTMLEditor::ToggleBrowseTemp()
{
		browseTempCopy=!browseTempCopy;
		options->setItemChecked(ID_BrowseTempFile, browseTempCopy);
		if (browseTempCopy)
				SetMainStatus(i18n("Browse Temporary File: On"));
		else
				SetMainStatus(i18n("Browse Temporary File: Off"));
}

void HTMLEditor::ToggleAutoSave()
{
		autoSaveOptions = !autoSaveOptions;
		options->setItemChecked(ID_AutoSaveOptions, autoSaveOptions);
		if (autoSaveOptions)
				SetMainStatus(i18n("Auto Save Options: On"));
		else
				SetMainStatus(i18n("Auto Save Options: Off"));
}

void HTMLEditor::ReadSettings()
{
		KConfig* config=kapp->getConfig();
		config->setGroup("Editor");
		editorFont=config->readFontEntry("Font", kapp->font());
		lowerCaseTags=config->readBoolEntry("Lowercase Tags", FALSE);
		closeAllTags=config->readBoolEntry("Close All Tags", TRUE);
		indentMode=config->readBoolEntry("Auto Indent", FALSE);
		useBackups=config->readBoolEntry("Create Backup Files", TRUE);
		browseTempCopy=config->readBoolEntry("Browse Temporary File", TRUE);
		browser=config->readEntry("External Browser", "/usr/local/bin/netscape");
		templatePath=config->readEntry("Template Path", kapp->kde_datadir() + "/" + kapp->appName() + "/templates");
		pixmapPrinting=config->readBoolEntry( "WYSIWYG Printing", FALSE);
		palette[HTMLEdit::ANCHOR]=config->readColorEntry("Anchor Tags", &TAG_COLORS[HTMLEdit::ANCHOR]);
		palette[HTMLEdit::COMMENT]=config->readColorEntry("Comment Tags", &TAG_COLORS[HTMLEdit::COMMENT]);
		palette[HTMLEdit::STYLE]=config->readColorEntry("Style Tags",	&TAG_COLORS[HTMLEdit::STYLE]);
		palette[HTMLEdit::IMAGE]=config->readColorEntry("Image Tags",	&TAG_COLORS[HTMLEdit::IMAGE]);
		palette[HTMLEdit::SCRIPT]=config->readColorEntry("Script Tags",	&TAG_COLORS[HTMLEdit::SCRIPT]);
		palette[HTMLEdit::TABLE]=config->readColorEntry("Table Tags",	&TAG_COLORS[HTMLEdit::TABLE]);
		palette[HTMLEdit::QUOTED]=config->readColorEntry("Quoted Values",	&TAG_COLORS[HTMLEdit::QUOTED]);
		palette[HTMLEdit::DEFAULT]=config->readColorEntry("Default Color",	&TAG_COLORS[HTMLEdit::DEFAULT]);
		
		config->setGroup("Toolbars");
		showToolBar[mainToolBar]=config->readBoolEntry("Main Toolbar", TRUE);
		showToolBar[generalToolBar]=config->readBoolEntry("General Toolbar", TRUE);
		showToolBar[fontsToolBar]=config->readBoolEntry("Fonts Toolbar", FALSE);
		showToolBar[formsToolBar]=config->readBoolEntry("Forms Toolbar", FALSE);
		showToolBar[framesToolBar]=config->readBoolEntry("Frames Toolbar", FALSE);
		showToolBar[listsToolBar]=config->readBoolEntry("Lists Toolbar", FALSE);
		showToolBar[scriptsToolBar]=config->readBoolEntry("Scripts Toolbar", FALSE);
		showToolBar[tablesToolBar]=config->readBoolEntry("Tables Toolbar", FALSE);
		toolBarAlternateSwitching=config->readBoolEntry("Alternate Switching", TRUE);

		pluginList=new QPluginList();
		QString nameMask("Name%d");
		QString commandMask("Command%d");
		QString name;
		QString command;
		uint i=1;
	
		config->setGroup("Import Plugins");
		name.sprintf(nameMask, i);
		command.sprintf(commandMask, i);
		while (config->hasKey(name))
				{
						PluginInfo *plugin=new PluginInfo;
						plugin->name=config->readEntry(name, 0);
						plugin->commandLine=config->readEntry(command, 0);
						plugin->type=PluginInfo::IMPORT;
						pluginList->append(plugin);
						i++;
						name.sprintf(nameMask, i);
						command.sprintf(commandMask, i);
				};
	
		config->setGroup("Export Plugins");
		i=1;
		name.sprintf(nameMask, i);
		command.sprintf(commandMask, i);
		while (config->hasKey(name))
				{
						PluginInfo *plugin=new PluginInfo;
						plugin->name=config->readEntry(name, 0);
						plugin->commandLine=config->readEntry(command, 0);
						plugin->type=PluginInfo::EXPORT;
						pluginList->append(plugin);
						i++;
						name.sprintf(nameMask, i);
						command.sprintf(commandMask, i);
				};
	
		config->setGroup("Filter Plugins");
		i=1;
		name.sprintf(nameMask, i);
		command.sprintf(commandMask, i);
		while (config->hasKey(name))
				{
						PluginInfo *plugin=new PluginInfo;
						plugin->name=config->readEntry(name, 0);
						plugin->commandLine=config->readEntry(command, 0);
						plugin->type=PluginInfo::FILTER;
						pluginList->append(plugin);
						i++;
						name.sprintf(nameMask, i);
						command.sprintf(commandMask, i);
				};
	
		config->setGroup("General");
		QSize size(500, 300);
		resize(config->readSizeEntry("Geometry", &size));
		autoSaveOptions=config->readBoolEntry("Auto Save Options", FALSE);
		maxRecentCount=config->readUnsignedNumEntry("Recent Files Count", 5);
		recentFilesList=new QStrList();
		config->readListEntry("Recent Files", *recentFilesList, ':');
		while (recentFilesList->count()>maxRecentCount)
				recentFilesList->removeFirst();

		config->setGroup("Directory Tree");
		htmlFilter=config->readEntry("HTML Files", HTML_MASK);
		textFilter=config->readEntry("Text Files", TEXT_MASK);
		imageFilter=config->readEntry("Image Files", IMAGE_MASK);
		backupFilter=config->readEntry("Backup Files", BACKUP_MASK);
		treeRoot=config->readEntry("Tree Root", "/");
		showAllFiles=config->readBoolEntry("Show All Files", FALSE);
		treeWidth=config->readNumEntry("Width", -1);

		//read toolbar structure
		QString globalConfig=kapp->kde_configdir()+"/webmaker-toolbarrc";
		QString localConfig=kapp->localconfigdir()+"/webmaker-toolbarrc";
		KConfig *toolBarConfig=new KConfig(globalConfig, localConfig);
		toolBarEntryList=new QToolBarEntryList();
		QStrList *tempList=new QStrList();
		ToolBarEntry *entry;
		QString groupMask("Toolbar%d");
		QString keyMask("Button%d");
		QString group, key;
		int groupNum;
		uint keyNum;
		for (groupNum=generalToolBar; groupNum<=tablesToolBar; groupNum++)
				{
						group.sprintf(groupMask, groupNum);
						toolBarConfig->setGroup(group);
						keyNum=1;
						key.sprintf(keyMask, keyNum);
						while (toolBarConfig->hasKey(key))
								{
										toolBarConfig->readListEntry(key, *tempList, ',');
										entry=new ToolBarEntry;
										entry->tag=tempList->first();
										entry->toolTip=tempList->next();
										entry->pixmap=tempList->next();
										entry->action=tempList->next();
										entry->toolBarID=groupNum;
										toolBarEntryList->append(entry);
										tempList->clear();
										keyNum++;
										key.sprintf(keyMask, keyNum);
								};
				};
		delete tempList;
		delete toolBarConfig;
}

void HTMLEditor::SaveSettings()
{
		SaveSettings(FALSE);
}

void HTMLEditor::SaveSettings(bool isExit)
{
		KConfig* config = kapp->getConfig();

		if (autoSaveOptions || !isExit)
				{
						config->setGroup("Editor");
						config->writeEntry("Font", editorFont);
						config->writeEntry("Lowercase Tags", lowerCaseTags);
						config->writeEntry("Close All Tags", closeAllTags);
						config->writeEntry("Auto Indent", indentMode);
						config->writeEntry("Create Backup Files", useBackups);
						config->writeEntry("Browse Temporary File", browseTempCopy);
						config->writeEntry("External Browser", browser);
						config->writeEntry("Template Path", templatePath);
						config->writeEntry("WYSIWYG Printing", pixmapPrinting);
						config->writeEntry("Anchor Tags", palette[HTMLEdit::ANCHOR]);
						config->writeEntry("Comment Tags", palette[HTMLEdit::COMMENT]);
						config->writeEntry("Style Tags",	palette[HTMLEdit::STYLE]);
						config->writeEntry("Image Tags",	palette[HTMLEdit::IMAGE]);
						config->writeEntry("Script Tags",	palette[HTMLEdit::SCRIPT]);
						config->writeEntry("Table Tags",	palette[HTMLEdit::TABLE]);
						config->writeEntry("Quoted Values",	palette[HTMLEdit::QUOTED]);
						config->writeEntry("Default Color",	palette[HTMLEdit::DEFAULT]);
	
						config->setGroup("Toolbars");
						config->writeEntry("Main Toolbar", showToolBar[mainToolBar]);
						config->writeEntry("General Toolbar", showToolBar[generalToolBar]);
						config->writeEntry("Fonts Toolbar", showToolBar[fontsToolBar]);
						config->writeEntry("Forms Toolbar", showToolBar[formsToolBar]);
						config->writeEntry("Frames Toolbar", showToolBar[framesToolBar]);
						config->writeEntry("Lists Toolbar", showToolBar[listsToolBar]);
						config->writeEntry("Scripts Toolbar", showToolBar[scriptsToolBar]);
						config->writeEntry("Tables Toolbar", showToolBar[tablesToolBar]);
						config->writeEntry("Alternate Switching", toolBarAlternateSwitching);
						config->setGroup("General");
						config->writeEntry("Auto Save Options", autoSaveOptions);
						config->writeEntry("Recent Files Count",	maxRecentCount);
		
						config->setGroup("Directory Tree");
						config->writeEntry("HTML Files", htmlFilter);
						config->writeEntry("Text Files", textFilter);
						config->writeEntry("Image Files", imageFilter);
						config->writeEntry("Backup Files", backupFilter);
						config->writeEntry("Tree Root", treeRoot);
						config->writeEntry("Show All Files", showAllFiles);
						keys->writeSettings(config);
				}
		
		config->setGroup("Directory Tree");
		config->writeEntry("Width", MainWidget->separatorPos());

		config->setGroup("General");
		config->writeEntry("Geometry", size());
		config->writeEntry("Recent Files", *recentFilesList, ':');
		
		config->sync();
}

void HTMLEditor::Help()
{
		kapp->invokeHTMLHelp("webmaker/index.html", "");
}

/*====================Support routines===============================*/

void HTMLEditor::ProcessToolBar(int buttonID)
{
		ToolBarEntry *entry=toolBarEntryList->at(buttonID);
		if (entry->action=="insertTag")
				{
						int pos=entry->tag.find(' ');
						if (pos==-1)
								Editor->insertTag(entry->tag);
						else
								{
										QString tag(entry->tag.left(pos));
										QString parameters(entry->tag.right(entry->tag.length()-pos));
										Editor->insertTag(tag, parameters);
								};
				}
		else if (entry->action=="editTag")
				{
						int pos=entry->tag.find(' ');
						if (pos==-1)
								Editor->insertTagWithEdit(entry->tag);
						else
								{
										QString tag(entry->tag.left(pos));
										QString parameters(entry->tag.right(entry->tag.length()-pos));
										Editor->insertTagWithEdit(tag, parameters);
								};
				}
		else if (entry->action=="insertEntity")
				Editor->insertEntity(entry->tag);
};

void HTMLEditor::FontFace(const char *fontName)
{
		QString s(" face=\"");
		s+=fontName;
		s+="\"";
		if (Editor->getTag().isEmpty())
				Editor->insertTag("FONT", s);
		else
				{
						QString tag(Editor->getTagElement());
						if (tag=="FONT")
								{
										tag=Editor->getTag();
										QRegExp mask("\\sface\\s*=\\s*\"[^\"]*\"", FALSE);
										if (tag.contains(mask))
												tag.replace(mask, s);
										else
												tag+=s;
										Editor->updateTag(tag);
								}
						else
								return;
				};
}

void HTMLEditor::FontSize(const char *fontSize)
{
		QString s(" size=\"");
		s+=fontSize;
		s+="\"";
		if (Editor->getTag().isEmpty())
				Editor->insertTag("FONT", s);
		else
				{
						QString tag(Editor->getTagElement());
						if (tag=="FONT")
								{
										tag=Editor->getTag();
										QRegExp mask("\\ssize\\s*=\\s*\"?[+-]?[0-7]\"?", FALSE);
										if (tag.contains(mask))
												tag.replace(mask, s);
										else
												tag+=s;
										Editor->updateTag(tag);
								}
						else
								return;
				};
}

void HTMLEditor::SetMainStatus(QString s)
{
		StatusBarTimer->stop();
		statusBar()->changeItem(s.data(),ID_STATUS_MAIN);
		StatusBarTimer->start(10000,TRUE);
}

// session managment functions:
void HTMLEditor::saveProperties(KConfig* config)
{
		HTMLEdit *editor;
		int total=EditorsList->count();
		config->writeEntry("totalEditors", total);
		QString superGroup=config->group();

		int current=0;
		for (editor=EditorsList->first(); editor!=0; editor=EditorsList->next())
				{
						QString groupno;
						groupno.setNum(current);
						config->setGroup(superGroup+QString("editorcontents-")+groupno);

						QString fullname;
						// make editor->getName() a full pathname:
						if (editor->getName().data()[0] != '/')
								{
										char* s = new char[1800];
										fullname =(getcwd(s, 1800));
										fullname +="/";
										delete [] s;
								};
						fullname += editor->getName().data();

						config->writeEntry("filename",fullname);
						config->writeEntry("modified",editor->isModified());
						// check here b/c readProperties will try to load [total] groups:
						if (strcmp(editor->getName(), i18n("Untitled"))==0 &&	!editor->isModified())
								continue;

						if (editor->isModified())
								{
										char *tempname = (char*)kapp->tempSaveName(fullname);
										editor->saveasfile(tempname);
										free(tempname);
								};
						current+=1; // check next editor in loop
				};
}

void HTMLEditor::readProperties(KConfig* config)
{
		int total=config->readNumEntry("totalEditors");
		QString superGroup=config->group();

		for (int current=0; current<total; current++)
				{
						QString groupno;
						groupno.setNum(current);
						config->setGroup(superGroup+QString("editorcontents-")+groupno);
						if (current) NewFile(); // first window has already be opened in InitMainWindget()
						QString filename=config->readEntry("filename");
						bool modified=config->readBoolEntry("modified", true);
						QString loadname=filename;
						if (modified)
								{
										// check for recover file with "unsaved" changes:
										bool recoverFound;
										const char *recoverName=kapp->checkRecoverFile(filename.data(),
																																																									recoverFound);
										if (recoverFound)
												loadname=recoverName; // loadname will be the file loaded
										else
												modified=false; // obviously, the changes are lost ;-(
								};
						if (strcmp(filename.data(), i18n("Untitled"))==0 && !modified)
								continue;

						int result=Editor->loadFile(loadname, KEdit::OPEN_READWRITE);
						if (result!=KEdit::KEDIT_OK) continue; //    !!!

						Editor->setName(filename);
						if (modified) Editor->setModified();
				};
	
		SetMainStatus(i18n("Session restored."));
		UpdateLineColStatus();
}

void HTMLEditor::RefreshMainStatus()
{
		statusBar()->changeItem("",ID_STATUS_MAIN);
}

void HTMLEditor::UpdateLineColStatus()
{
		QString linenumber;
		linenumber.sprintf(i18n("Line: %d Col: %d"), Editor->currentLine() + 1, Editor->currentColumn() + 1);
		statusBar()->changeItem(linenumber.data(), ID_STATUS_LINE_COL);
}

void HTMLEditor::ToggleInsStatus()
{
		if(Editor->isOverwriteMode())
				{
						statusBar()->changeItem("OVR",ID_STATUS_INS);
				}
		else
				{
						statusBar()->changeItem("INS",ID_STATUS_INS);
				}
}

void HTMLEditor::NextWindow()
{
		int id=EditorsList->findRef(Editor)+1;
		if (EditorsList->count()>id)
				ChangeWindow(id);
		else
				ChangeWindow(0);
}

void HTMLEditor::PreviousWindow()
{
		int id=EditorsList->findRef(Editor)-1;
		if (id>=0)
				ChangeWindow(id);
		else
				ChangeWindow(EditorsList->count()-1);
}

void HTMLEditor::ChangeWindow(int id)
{
		Editors->showPage(EditorsList->at(id));
}

void HTMLEditor::SetCurrentEditor(QWidget *editor)
{
	 Editor=(HTMLEdit*)editor;
		//Editor->raise();
		Editor->setFocus();
		
		UpdateCaption();
		UpdateLineColStatus();
}

void HTMLEditor::UpdateCaption()
{
		QString title;
		title.sprintf("%s: %s", APP_NAME, Editor->getName().data());
		setCaption(title);
		Editors->setTabLabel(Editor, QFileInfo(Editor->getName()).fileName());
}

void HTMLEditor::UpdateMenuBar()
{
		//Update "Windows" menu
		windows->clear();
		QString itemText(1024);
		HTMLEdit *editor=EditorsList->first();
		while (editor)
				{
						itemText.sprintf("%d: %s", EditorsList->at()+1, editor->getName().data());
						windows->insertItem(itemText);
						editor=EditorsList->next();
				};
		
		//All stuff is completely wrong :-((( Commenting out...
		//Must handle enabling/disabling of items in menus
		
		/*
				uint i, count;
		
				if (Editor->getTag().isEmpty())
				{
				tools->setItemEnabled(ID_InsertTag, TRUE);
				tools->setItemEnabled(ID_EditTag, FALSE);
				tools->setItemEnabled(ID_TableTemplate, TRUE);
				EditorMenu->setItemEnabled(ID_EditorInsertTag, TRUE);
				EditorMenu->setItemEnabled(ID_EditorEditTag, FALSE);
				count=tags->count();
				for (i=0; i<count; i++)
				{
				tags->setItemEnabled(tags->idAt(i), TRUE);
				};
				}
				else
				{
				tools->setItemEnabled(ID_InsertTag, FALSE);
				tools->setItemEnabled(ID_EditTag, TRUE);
				tools->setItemEnabled(ID_TableTemplate, FALSE);
				EditorMenu->setItemEnabled(ID_EditorInsertTag, FALSE);
				EditorMenu->setItemEnabled(ID_EditorEditTag, TRUE);
				count=tags->count();
				for (i=0; i<count; i++)
				{
				tags->setItemEnabled(tags->idAt(i), FALSE);
				};
				};
				if (QApplication::clipboard()->text())
				{
				edit->setItemEnabled(ID_Paste, TRUE);
				EditorMenu->setItemEnabled(ID_EditorPaste, TRUE);
				}
				else
				{
				edit->setItemEnabled(ID_Paste, FALSE);
				EditorMenu->setItemEnabled(ID_EditorPaste, FALSE);
				};
				if (Editor->markedText().isEmpty())
				{
				edit->setItemEnabled(ID_Cut, FALSE);
				edit->setItemEnabled(ID_Copy, FALSE);
				EditorMenu->setItemEnabled(ID_EditorCut, FALSE);
				EditorMenu->setItemEnabled(ID_EditorCopy, FALSE);
				}
				else
				{
				edit->setItemEnabled(ID_Cut, TRUE);
				edit->setItemEnabled(ID_Copy, TRUE);
				EditorMenu->setItemEnabled(ID_EditorCut, TRUE);
				EditorMenu->setItemEnabled(ID_EditorCopy, TRUE);
				};
				if (Editor->isModified() || Editor->getName()!=i18n("Untitled"))
				{
				file->setItemEnabled(ID_Save, TRUE);
				file->setItemEnabled(ID_SaveAs, TRUE);
				file->setItemEnabled(ID_Print, TRUE);
				efile->setItemEnabled(ID_EditorSave, TRUE);
				efile->setItemEnabled(ID_EditorSaveAs, TRUE);
				}
				else
				{
				file->setItemEnabled(ID_Save, FALSE);
				file->setItemEnabled(ID_SaveAs, FALSE);
				file->setItemEnabled(ID_Print, FALSE);
				efile->setItemEnabled(ID_EditorSave, FALSE);
				efile->setItemEnabled(ID_EditorSaveAs, FALSE);
				};
				file->setItemEnabled(ID_SaveAll, FALSE);
				HTMLEdit *editor=EditorsList->first();
				while (editor)
				{
				if (editor->isModified() || editor->getName()!=i18n("Untitled"))
				{
				file->setItemEnabled(ID_SaveAll, TRUE);
				break;
				};
				editor=EditorsList->next();
				}
		*/
}

void HTMLEditor::AddRecentFile(const char* fileName)
{
  if (recentFilesList->find(fileName)!=-1) return; // it's already there

  if (recentFilesList->count() < maxRecentCount)
    recentFilesList->append(fileName);
  else
				{
						recentFilesList->removeFirst();
						recentFilesList->append(fileName);
				};

  recent->clear();
  for (uint i=0; i < recentFilesList->count(); i++)
    recent->insertItem(recentFilesList->at(i));
}

void HTMLEditor::Loading()
{
		SetMainStatus("Loading...");
}

void HTMLEditor::Saving()
{
		SetMainStatus("Saving...");
}

void HTMLEditor::DefaultHandler()
{
		KMsgBox::message(this, i18n("Sorry!"), i18n("Sorry, feature is not implemented yet."));
}
